/**
 * Initialize Application
 */
var app = angular.module('gr.ant1.app', ['gr.ant1.app.config', 'ionic', 'ngCordova', 'ngCordovaOauth', 'ngMessages', 'angular-cache', 'pascalprecht.translate', 'bt.jsonValidator', 'schemaInstantiator'])
    .run(function (Menu, $rootScope, $state, $log, $cordovaGoogleAnalytics, $ionicPlatform, $timeout, $window, ConnectivityMonitor, NOTIFY_CONFIG, GA_CONFIG) {
        /**
         * Sets the Application Logging
         */
        Menu.getMenu().then(function (result) {
            /**
             * Used For Application Logging
             */
            NOTIFY_CONFIG.ACTIVE = result.data.logging ? NOTIFY_CONFIG.NAME_SENTRY : '';

        }).catch(function (reason) {
            /**
             * Call the getMenuLocal which will fetch the local Configuration JSON
             */
            Menu.getMenuLocal().then(function (resultLocal) {
                /**
                 * Used For Application Logging
                 */
                NOTIFY_CONFIG.ACTIVE = resultLocal.data.logging ? NOTIFY_CONFIG.NAME_SENTRY : '';

            }).catch(function (reasonLocal) {

            });
        });

        /**
         * Ionic Platform ready
         */
        $ionicPlatform.ready(function () {
            if (window.cordova && window.cordova.plugins.Keyboard) {
                /**
                 * Hide the accessory bar by default (remove this to show the
                 * accessory bar above the keyboard for form inputs
                 */
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);

                /**
                 * Don't remove this line unless you know what you are doing. It stops the viewport
                 * from snapping when text inputs are focused. Ionic handles this internally for
                 * a much nicer keyboard experience.
                 */
                cordova.plugins.Keyboard.disableScroll(false);
            }
            if (window.StatusBar) {
                StatusBar.styleDefault();
            }

            /**
             * Sets the full screen
             */
            ionic.Platform.fullScreen(true, true);

            /**
             * Start watching for device network connectivity changes
             */
            ConnectivityMonitor.startWatching();

            /**
             * Generic Event listener for the device back button
             */
            document.addEventListener("backbutton", $rootScope.genericBackButtonHandler, false);

            /**
             * Initialize Google Analytics
             */
            if ($window.ga) {
                $window.ga.startTrackerWithId(GA_CONFIG.TRACKING_ID, function (response) {
                    $window.ga.setAppVersion("2.0.0");
                    $window.ga.trackView('Initialize Application');
                    $window.ga.trackEvent("App", "Initialize", "Platform Ready");
                }, function (reason) {
                    //$log.warn("Could not initialize Google Analytics -> " + reason);
                });
            }
        });
    })
    .config(function ($stateProvider, $urlRouterProvider, $ionicConfigProvider, CacheFactoryProvider, CACHE_DEFAULTS) {
        /**
         * Set default values from 'config' for all $http calls
         */
        angular.extend(CacheFactoryProvider.defaults, CACHE_DEFAULTS);

        /**
         * Enable native scrolling
         *
         * NOTE: You can also enable it on a single view
         * by adding overflow-scroll='true' on your ion-content tag.
         */
        $ionicConfigProvider.scrolling.jsScrolling(false);

        /**
         * Or for only a single platform
         */
        /*if (ionic.Platform.isAndroid()) {
         $ionicConfigProvider.scrolling.jsScrolling(false);
         }*/

        /**
         * Set ionic cached view size globally
         */
        $ionicConfigProvider.views.maxCache(5);
        /**
         * Per platforms
         */
        //$ionicConfigProvider.platform.android.views.maxCache(5);
        //$ionicConfigProvider.platform.ios.views.maxCache(5);

        /**
         * Sets the swipe to go back
         */
        $ionicConfigProvider.views.swipeBackEnabled(false);

        /**
         * Application Routes
         */
        $stateProvider
            .state('app', {
                url: '/app',
                abstract: true,
                templateUrl: 'templates/menu/menu.html',
                controller: 'AppCtrl'
            })
            .state('app.home', {
                url: '/home',
                views: {
                    'menuContent': {
                        templateUrl: "templates/home/home.html",
                        controller: 'HomepageController'
                    }
                }
            })
            .state('app.schedule', {
                url: "/schedule",
                views: {
                    'menuContent': {
                        templateUrl: "templates/schedule/schedule.html",
                        controller: "ScheduleController"
                    }
                }
            })
            .state('app.show', {
                url: "/show/:id",
                views: {
                    'menuContent': {
                        templateUrl: "templates/shows/show-details.html",
                        controller: "ShowDetailsController"
                    }
                }
            })
            .state('app.shows', {
                url: "/shows",
                views: {
                    'menuContent': {
                        templateUrl: "templates/shows/shows.html",
                        controller: "ShowsController"
                    }
                }
            })
            .state('app.showsArchive', {
                url: "/shows/archive",
                views: {
                    'menuContent': {
                        templateUrl: "templates/shows/archive.html",
                        controller: "ShowsArchiveController"
                    }
                }
            })
            .state('app.showsTrailers', {
                url: "/shows/trailers",
                views: {
                    'menuContent': {
                        templateUrl: "templates/shows/trailers.html",
                        controller: "VideoTrailersController"
                    }
                }
            })
            .state('app.search', {
                url: "/search/:q",
                cache: false,
                views: {
                    'menuContent': {
                        templateUrl: "templates/shows/search.html",
                        controller: "SearchController"
                    }
                }
            });
        /**
         * Application Default route
         */
        $urlRouterProvider.otherwise('/app/home');
    });