angular.module('gr.ant1.app')
    /**
     * Main Application Controller
     */
    .controller('AppCtrl', function (Menu, NotifyFactory, $window, $ionicHistory, $log, $timeout, $rootScope, $scope, $stateParams, $state, $translate, $ionicLoading, $ionicSideMenuDelegate, $ionicSlideBoxDelegate, $ionicScrollDelegate, jsonValidator, InstantiatorService, CacheFactory, SERVICE_MENU, BASE_URL, NOTIFY_CONFIG) {
        /**
         * toggleLeftSideMenu
         *
         * Toggles the left side menu of the app
         */
        $scope.toggleLeftSideMenu = function () {
            $ionicSideMenuDelegate.toggleLeft();
        };

        $scope.$on('$ionicView.enter', function () {
            $scope.notificationsCount = 25;
            /**
             * Manually handle the ionic navigation history
             */
            var getViewHistory = $ionicHistory.viewHistory();
            $scope.historyHasBackView = getViewHistory.backView;
        });

        /**
         * This event will trigger, no matter it was the first load or a cached view.
         */
        $scope.$on('$ionicView.beforeEnter', function (states, scopes) {
            /**
             * Bring the configuration data for the application, menu, social media etc
             */
            $rootScope.checkCache(states, scopes, 'menuCache', BASE_URL.API_URL + SERVICE_MENU.URL, $scope, CacheFactory);
        });

        /**
         * Fetch view data from factory
         */
        $scope.updateViewData = function () {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Gets the menu
             */
            Menu.getMenu().then(function (result) {
                /**
                 * Used For Application Logging
                 */
                NOTIFY_CONFIG.ACTIVE = result.data.logging ? NOTIFY_CONFIG.NAME_SENTRY : '';

                $scope.menuItems = result.data.menu;
                $scope.appMetaData = result.data.meta;
                $scope.socialMedia = result.data.meta.social;

            }).catch(function (reason) {

                /**
                 * Call the getMenuLocal which will fetch the local Configuration JSON
                 */
                Menu.getMenuLocal().then(function (resultLocal) {
                    /**
                     * Used For Application Logging
                     */
                    NOTIFY_CONFIG.ACTIVE = resultLocal.data.logging ? NOTIFY_CONFIG.NAME_SENTRY : '';

                    $scope.menuItems = resultLocal.data.menu;
                    $scope.appMetaData = resultLocal.data.meta;
                    $scope.socialMedia = resultLocal.data.meta.social;

                }).catch(function (reasonLocal) {

                    /**
                     * Shows alert popup to inform the user, there are no available data
                     */
                    $scope.showAlertInvalidData($translate.instant('header_title_menu'), $translate.instant('alert_no_data'), '', $scope.doRefresh);
                });

            }).finally(function () {
                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };

        /**
         * Custom history function for Back button
         */
        $scope.customHistoryGoBack = function () {
            /**
             * Google Analytics Track View
             */
            if ($window.ga) {
                $window.ga.trackEvent("History", "Back From", $ionicHistory.currentTitle());
            }
            $ionicHistory.goBack();
        };

        /**
         * Opens window with a social page
         * ex.: Facebook, Twitter
         *
         * @param socialUrl
         */
        $scope.openSocialUrl = function (socialUrl) {
            window.open(socialUrl, '_system');
        };

        /**
         * Opens window with external url
         *
         * @param url
         */
        $scope.openExternalUrl = function (url) {
            window.open(url, '_system');
        };
        /**
         * function which returns the default fallback images when img src or srcset fails to load
         * @param target
         * @param className
         * @returns {string}
         */
        $window.onImageErrorFallback = function (target, className) {
            var width = $window.innerWidth;
            var imgURL = 'img/default/';

            if (width >= 320 && width <= 570) {
                switch (className) {
                    case 'app-home-carousel':
                        target.src = imgURL + 'noimage640x360.png';
                        break;
                    case 'app-home-episodes':
                        target.src = imgURL + 'noimage640x360.png';
                        break;
                    case 'app-shows-trailers':
                        target.src = imgURL + 'noimage640x360.png';
                        break;
                    case 'app-shows-details-cover':
                        target.src = imgURL + 'noimage640x360.png';
                        break;
                    case 'app-shows-details-episodes':
                        target.src = imgURL + 'noimage640x360.png';
                        break;
                    case 'app-shows-logo':
                        target.src = imgURL + 'noimage250x250.png';
                        break;
                }
            } else if (width > 570 && width <= 768) {
                switch (className) {
                    case 'app-home-carousel':
                        target.src = imgURL + 'noimage768x400.png';
                        break;
                    case 'app-home-episodes':
                        target.src = imgURL + 'noimage350x195.png';
                        break;
                    case 'app-shows-trailers':
                        target.src = imgURL + 'noimage768x400.png';
                        break;
                    case 'app-shows-details-cover':
                        target.src = imgURL + 'noimage768x400.png';
                        break;
                    case 'app-shows-details-episodes':
                        target.src = imgURL + 'noimage350x195.png';
                        break;
                    case 'app-shows-logo':
                        target.src = imgURL + 'noimage250x250.png';
                        break;
                }
            } else {
                switch (className) {
                    case 'app-home-carousel':
                        target.src = imgURL + 'noimage1024x310.png';
                        break;
                    case 'app-home-episodes':
                        target.src = imgURL + 'noimage310x174.png';
                        break;
                    case 'app-shows-trailers':
                        target.src = imgURL + 'noimage1024x400.png';
                        break;
                    case 'app-shows-details-cover':
                        target.src = imgURL + 'noimage1024x400.png';
                        break;
                    case 'app-shows-details-episodes':
                        target.src = imgURL + 'noimage310x174.png';
                        break;
                    case 'app-shows-logo':
                        target.src = imgURL + 'noimage250x250.png';
                        break;
                }
            }

        };

        /**
         * Query Search Text
         * @type {{search: {text: string}}}
         */
        $rootScope.searchQuery = {
            search: {text: ""}
        };
        /**
         * Sets the caller for the search
         * @type {string}
         */
        $rootScope.searchCaller = 'shows';
        /**
         * Implement the search bar functionality according to whether this is a search
         * for the ARCHIVE or the SHOWS
         */
        $rootScope.clearSearch = function () {
            $rootScope.searchQuery.search = {text: ""};
        };
        $rootScope.triggerSearch = function () {
            if ($rootScope.searchQuery.search.text.length < 3) {
                return false;
            } else {
                $state.go('app.search', {q: $rootScope.searchQuery.search.text});
            }
        };
    });