angular.module('gr.ant1.app')
/**
 * Home Page Controller
 */
    .controller('HomepageController', function (Homepage, HomepageSchema, $log, $window, $timeout, $rootScope, $scope, $stateParams, $state, $translate, $ionicHistory, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate, jsonValidator, InstantiatorService, $q, CacheFactory, BASE_URL, SERVICE_HOMEPAGE) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Homepage");

        /**
         * Where to start data
         * @type {number}
         */
        var start = 0;
        /**
         * How many tiles will show on every reload
         * @type {number}
         */
        var step = 6;
        /**
         * How much data need to add on each function call
         * @type {number}
         */
        var ending = start + step;
        /**
         * Maximum number of calls.
         * @type {number}
         */
        var last = 0;
        /**
         * To check the page ends last or not
         * @type {boolean}
         */
        $scope.moreDataCanBeLoaded = true;

        /**
         *
         * Declare and initialize namespace object to scope
         */
        $scope.ns = {};

        /**
         * Declare and initialize tabs object to scope
         *
         * @type {*[]}
         */
        $scope.ns.menu = [
            {'Name': $translate.instant('home_tab_latest_episodes')}
            , {'Name': $translate.instant('home_tab_recommended')}
            , {'Name': $translate.instant('home_tab_shows')}
        ];

        /**
         * Declare and initialize tab content array to scope
         *
         * @type {Array}
         */
        $scope.ns.content = [];

        /**
         * Declare and initialize carousel array to scope
         *
         * @type {Array}
         */
        $scope.carousel = [];

        /**
         * Workaround for slide-box
         */
        $scope.onTouch = function () {
            $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(false);
        };
        $scope.onRelease = function () {
            $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(true);
        };
        $scope.nextSlide = function () {
            $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').next(500);
        };
        $scope.previousSlide = function () {
            $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').previous(500);
        };

        /**
         * This event will trigger, no matter it was the first load or a cached view.
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            $rootScope.checkCache(states, scopes, 'homepageCache', BASE_URL.API_URL + SERVICE_HOMEPAGE.RECOMMENDED, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Home");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory
         */
        $scope.updateViewData = function () {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Promises array
             *
             * @type {{schema_carousel: *, schema_episodes: *, schema_recommended: *, carousel: *, episodes: *, recommended: *}}
             */
            var promises = {
                /**
                 * Gets the carousel schema object
                 */
                schema_carousel: HomepageSchema.getCarousel().then(function (value) {
                    return InstantiatorService.instantiate(value.data).items;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the episodes schema object
                 */
                schema_episodes: HomepageSchema.getLatestEpisodes().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the recommended schema object
                 */
                schema_recommended: HomepageSchema.getRecommended().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the carousel
                 */
                carousel: Homepage.getCarousel().then(function (value) {
                    return value.data.items;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the latest episodes
                 */
                episodes: Homepage.getLatestEpisodes().then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the recommended
                 */
                recommended: Homepage.getRecommended().then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return [];
                })
            };

            /**
             * Construct all data for categories and shows
             */
            $q.all(promises).then(function (result) {
                /**
                 * Sets scope carousel array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                $scope.carousel = angular.merge([], result.schema_carousel, result.carousel);
                $scope.carousel = reorderCarousel($scope.carousel);
                /**
                 * Sets scope episodes array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                $scope.ns.menu[0].items = angular.merge([], result.schema_episodes, result.episodes);

                /**
                 * Sets scope recommended array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                $scope.ns.menu[1].items = angular.merge([], result.schema_recommended, result.recommended);

                /**
                 * Enables sliding in slidebox
                 */
                $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(true);

                /**
                 * Sets the default tab content
                 */
                setSlideContent($ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').currentIndex());

            }).catch(function (reason) {
                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_home'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };

        /**
         * Change the slide content by tab index
         *
         * @param index
         */
        function setSlideContent(index) {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Scroll to top the list
             */
            $ionicScrollDelegate.scrollTop(true);

            /**
             * Initialize the content data array
             * @type {*}
             */
            $scope.ns.content = [];
            $scope.ns.content.length = 0;

            /**
             * Where to start data
             * @type {number}
             */
            start = 0;
            /**
             * How much data need to add on each function call
             * @type {number}
             */
            ending = start + step;
            /**
             * Last data content
             */
            last = angular.copy($scope.ns.menu[index].items).length;
            /**
             * To check the page ends last or not
             * @type {boolean}
             */
            $scope.moreDataCanBeLoaded = true;

            /**
             * Load more data
             */
            $scope.loadMore();
        }

        /**
         * Slide drag left
         * Disables sliding in slidebox when is active the second slide
         */
        /*$scope.dragLeft = function () {
         if ($ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').currentIndex() === 1) {
         $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(false);
         }
         };*/
        /**
         * Slide drag right
         * Enables sliding in slidebox when is active the second slide
         */
        /*$scope.dragRight = function () {
         if ($ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').currentIndex() === 1) {
         $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(true);
         }
         };*/

        /**
         * Slide change event handler
         * @param index
         */
        $scope.slideHasChanged = function (index) {
            /**
             * Redirect to Shows page
             * and sets the slider to the previous tab
             */
            if (index === 2) {
                $state.go('app.shows');
                $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').previous();
            } else {
                if (index === 0) $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(true);
                if (index === 1) $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').enableSlide(false);
                setSlideContent(index);
            }
        };

        /**
         * Update slider view
         */
        $scope.updateSlider = function () {
            $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').update();
        };

        /**
         * Load more with infinite-scroll
         */
        $scope.loadMore = function () {
            var data = $scope.ns.menu[$ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').currentIndex()];
            if (!angular.isUndefined(data)) {
                var items = angular.copy(data.items);
                if (items != undefined) {
                    var current_index = $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').currentIndex();

                    /**
                     * Sets the content data array
                     * @type {*}
                     */
                    if (last <= ending) ending = last;
                    for (start; start < ending; start++) {
                        items[start].template = 'templates/home/partials/video.html';
                        if (current_index === 1) {
                            items[start].OwnerTitle = items[start].Title;
                            items[start].template = items[start].IsShow ? 'templates/home/partials/show.html' : 'templates/home/partials/video.html';
                        }
                        $scope.ns.content.push(items[start]);
                    }

                    /**
                     * Check if you have reached at bottom
                     */
                    if (ending >= last) {
                        $scope.moreDataCanBeLoaded = false;
                    }
                    ending = start + step;

                    /**
                     * Broadcast scroll event
                     */
                    $scope.$broadcast('scroll.infiniteScrollComplete');

                    /**
                     * Updates view
                     */
                    $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').update();
                    $timeout(function () {
                        $ionicSlideBoxDelegate.$getByHandle('app-inline-tabs-content').update();

                        /**
                         * Hides the loader
                         */
                        $ionicLoading.hide();
                    }, 500);
                }
            }
        };


         reorderCarousel = function(carousel) {
            if(carousel.length > 2){
                var tempArray = [];
                for(var i=1; i< carousel.length; i++){
                    tempArray.push(carousel[i]);
                }
                tempArray.push(carousel[0]);
                carousel = tempArray;
            }
            return carousel;
        }



        $scope.$on('$ionicView.loaded', function (states, scopes) {
            /*======================================*/
            /*              CAROUSEL                */
            /*======================================*/
            $window.isCarouselStart = true;
            $window.isCarouselInit = false;
            $scope.carouselData = {};
            $scope.carouselData.sliderOptions = {
                initialSlide: 0,
                pagination: false,
                loop: true,
                onInit: function (slider) {
                    $scope.carouselData.swiper = slider;
                    $scope.carouselAutoPlay();
                    slider.updateActiveIndex();
                },
                onSlideChangeStart: function (slider) {
                    slider.update();
                    clearTimeout($rootScope.homeCarouselTimeoutID);
                },
                onSlideChangeEnd: function (slider) {
                    slider.update();
                },
                onTouchStart: function (slider) {
                    slider.update();
                }
            };

            $scope.carouselAutoPlay = function() {
                var intervalInSeconds = 5;

                $scope.carouselData.swiper.update();
                if($window.isCarouselInit) {
                    $scope.carouselData.swiper.slideTo(1);
                    $window.isCarouselInit = false;
                } else {
                    $scope.carouselData.swiper.slideNext();
                }

                if($rootScope.homeCarouselTimeoutID) clearTimeout($rootScope.homeCarouselTimeoutID);
                $rootScope.homeCarouselTimeoutID = setTimeout($scope.carouselAutoPlay, intervalInSeconds * 1000);
            }


            $scope.nextCarouselSlide = function () {
                $scope.carouselData.swiper.update();
                clearTimeout($scope.timeOutID);
                $scope.carouselData.swiper.slideNext();
            };

            $scope.prevCarouselSlide = function () {
                $scope.carouselData.swiper.update();
                clearTimeout($scope.timeOutID);
                $scope.carouselData.swiper.slidePrev();
            };


        });
    });