/**
 * Created by vanger on 12/08/2016.
 */
angular.module('gr.ant1.app')
    /**
     * Schedule Controller
     */
    .controller('ScheduleController', function (Schedule, ScheduleSchema, $log, $window, $timeout, $rootScope, $scope, $stateParams, $translate, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate, $ionicHistory, $filter, jsonValidator, InstantiatorService, $q, CacheFactory, BASE_URL, SERVICE_SCHEDULE) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Schedule");

        /**
         * Declare and initialize items object to scope
         * @type {{}}
         */
        $scope.items = {};

        /**
         * Declare and initialize tabs array to scope
         * @type {Array}
         */
        $scope.items.tabs = [];

        /**
         * Declare and initialize tab content array to scope
         * @type {Array}
         */
        $scope.items.content = [];

        /**
         * Declare and initialize tabs schema object to scope
         * @type {Array}
         */
        $scope.schema_tabs = [];

        /**
         * Workaround for slide-box
         */
        $scope.onTouch = function () {
            $ionicSlideBoxDelegate.enableSlide(false);
        };
        $scope.onRelease = function () {
            $ionicSlideBoxDelegate.enableSlide(true);
        };
        $scope.nextSlide = function () {
            $ionicSlideBoxDelegate.next(500);
        };
        $scope.previousSlide = function () {
            $ionicSlideBoxDelegate.previous(500);
        };

        /**
         * On Enter view event
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            $rootScope.checkCache(states, scopes, 'scheduleCache', BASE_URL.API_URL + SERVICE_SCHEDULE.URL, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Schedule");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory
         */
        $scope.updateViewData = function () {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Promises array
             *
             * @type {{schema_trailers: *, trailers: *}}
             */
            var promises = {
                /**
                 * Gets the schedule tabs schema object
                 */
                schema_tabs: ScheduleSchema.getTabs().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the schedule tabs
                 */
                tabs: Schedule.getTabs().then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return [];
                })
            };

            /**
             * Construct all data for categories and shows
             */
            $q.all(promises).then(function (result) {
                /**
                 * Sets scope schedule tabs array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                $scope.items.tabs = angular.merge([], result.schema_tabs, $scope.refactorScheduleData(result.tabs));

                /**
                 * Enables sliding in slidebox
                 */
                $ionicSlideBoxDelegate.enableSlide(true);

                /**
                 * Sets the default tab content
                 */
                setSlideContent($ionicSlideBoxDelegate.currentIndex());

            }).catch(function (reason) {
                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_schedule'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };

        /**
         * Change the slide content by tab index
         *
         * @param index
         */
        function setSlideContent(index) {
            $scope.items.content = $scope.items.tabs[index].items;

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackEvent("Schedule", "Day", $scope.items.tabs[index].meta.WeekDay + ' - ' + $scope.items.tabs[index].meta.WeekDate);

            /**
             * Updates view
             */
            $ionicSlideBoxDelegate.update();
            $timeout(function () {
                $ionicSlideBoxDelegate.update();
            }, 500);
        }

        /**
         * Slide change event handler
         * @param index
         */
        $scope.slideHasChanged = function (index) {
            setSlideContent(index);
        };

        /**
         * Function to refactor the Schedule data and find the active day and live show
         * @param all_data
         * @returns {Array}
         */
        $scope.refactorScheduleData = function (all_data) {
            var tempData = [];
            angular.forEach(all_data, function (data, key) {
                var temp = [];
                var today = $filter('date')(new Date(), 'MM/dd');
                var now = $filter('date')(new Date(), 'yyyy-MM-dd HH:mm');

                if (data.meta.WeekDate === today) {
                    data.meta.active = true;
                } else {
                    data.meta.active = false;
                }
                angular.forEach(data.items, function (item, key) {
                    var tempStartTime = $filter('date')(new Date(item.StartDate), 'yyyy-MM-dd HH:mm');
                    var tempEndTime = $filter('date')(new Date(item.EndDate), 'yyyy-MM-dd HH:mm');
                    if (now >= tempStartTime && now < tempEndTime) {
                        item.isLive = true;
                    } else {
                        item.isLive = false;
                    }
                }, temp);
                this.push(data);
            }, tempData);
            return tempData;
        };

    });
