/**
 * Created by vanger on 12/08/2016.
 */
angular.module('gr.ant1.app')
/**
 * Show Details Controller
 */
    .controller('ShowDetailsController', function (Shows, ShowsSchema, $log, $window, $timeout, $rootScope, $scope, $stateParams, $ionicHistory, $translate, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate, jsonValidator, InstantiatorService, CacheFactory, $q, BASE_URL, SERVICE_SHOWS) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Show Details");

        /**
         * The id of the show
         * @type {string}
         */
        var show_id = '';
        /**
         * The current page of episodes for the selected show
         * @type {number}
         */
        var page = 1;
        /**
         * The total pages of episodes for the selected show
         * @type {number}
         */
        var total_pages = 0;

        /**
         * The Schema Episodes object
         * @type {{}}
         */
        var schema_episodes = {};

        /**
         * To check the page ends, last or not
         * @type {boolean}
         */
        $scope.moreDataCanBeLoaded = true;

        /**
         * Declare and initialize show details object to scope
         * @type {Array}
         */
        $scope.showDetails = {};

        /**
         * Declare and initialize show episodes array to scope
         * @type {Array}
         */
        $scope.showEpisodes = [];

        /**
         * Declare and initialize show episodes items array to scope
         * @type {Array}
         */
        $scope.showEpisodesItems = [];

        /**
         * Declare and initialize shown item object to scope
         * @type {{}}
         */
        $scope.shownItem = {};

        /**
         * On Enter view event
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            $rootScope.checkCache(states, scopes, 'showsCache', BASE_URL.API_URL + SERVICE_SHOWS.URL, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Show Details");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory using cue promise
         */
        $scope.updateViewData = function () {
            /**
             * Sets id of the show
             */
            show_id = $stateParams.id;

            /**
             * Resets the current page of episodes for the selected show
             * @type {number}
             */
            page = 1;

            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Promises array
             *
             * @type {{schema_details: *, schema_episodes: *, details: *, episodes: *}}
             */
            var promises = {
                /**
                 * Gets the episodes for show schema object
                 */
                schema_details: ShowsSchema.getDetails().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the episodes for show schema object
                 */
                schema_episodes: ShowsSchema.getEpisodesForShow().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets a show details by id
                 */
                details: Shows.getShowDetailsById(show_id).then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return {};
                }),
                /**
                 * Gets all episodes for the selected show
                 */
                episodes: Shows.getEpisodesForShow(show_id, page).then(function (value) {
                    return value.data;
                }, function (reason) {
                    return {};
                })
            };

            /**
             * Construct all data for show details and episodes for selected show
             */
            $q.all(promises).then(function (result) {
                /**
                 * Stores the schema episodes object
                 * @type {*}
                 */
                schema_episodes = angular.copy(result.schema_episodes);

                /**
                 * Sets show details scope object
                 * Merge schema objects with requested objects for always right structure
                 * @type {Object}
                 */
                $scope.showDetails = angular.merge({}, result.schema_details, result.details);

                /**
                 * Google Analytics Track Event
                 */
                if ($window.ga) $window.ga.trackEvent("Show", $stateParams.id, $scope.showDetails.Title);

                /**
                 * Sets show episodes scope array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Object}
                 */
                $scope.showEpisodes = angular.merge([], result.schema_episodes, result.episodes.data);

                /**
                 * Initialize show episode items array
                 * @type {Array}
                 */
                $scope.showEpisodesItems = [];

                /**
                 * Add data items to scope array
                 * @type {number}
                 */
                for (var i = 0; i < $scope.showEpisodes.length; i++) {
                    $scope.showEpisodesItems.push($scope.showEpisodes[i]);
                }

                /**
                 * Resets the total pages of episodes for the selected show
                 */
                total_pages = result.episodes.total_pages;

                /**
                 * Resets the flag
                 */
                $scope.moreDataCanBeLoaded = true;

                /**
                 * Check if you have reached at bottom
                 */
                if (page >= total_pages) $scope.moreDataCanBeLoaded = false;

                /**
                 * Increment the current page
                 */
                page++;

            }).catch(function (reason) {
                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_shows'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };

        /**
         * Load more for infinite-scroll with paging request
         */
        $scope.loadMore = function () {
            if (page <= total_pages) {
                /**
                 * Promises array
                 * @type {{episodes: *}}
                 */
                var promises = {
                    /**
                     * Gets all episodes for the selected show
                     */
                    episodes: Shows.getEpisodesForShow(show_id, page).then(function (value) {
                        return value.data;
                    }, function (reason) {
                        return {};
                    })
                };
                /**
                 * Construct all data for show details and episodes for selected show
                 */
                $q.all(promises).then(function (result) {
                    /**
                     * Sets show episodes scope array
                     * Merge schema objects with requested objects for always right structure
                     * @type {Object}
                     */
                    $scope.showEpisodes = angular.merge([], schema_episodes, result.episodes.data);

                    /**
                     * Add data items to scope array
                     * @type {number}
                     */
                    for (var i = 0; i < $scope.showEpisodes.length; i++) {
                        $scope.showEpisodesItems.push($scope.showEpisodes[i]);
                    }

                    /**
                     * Check if you have reached at bottom
                     */
                    if (page >= total_pages) $scope.moreDataCanBeLoaded = false;

                    /**
                     * Increment the current page
                     */
                    page++;

                    /**
                     * Last page
                     */
                    total_pages = result.episodes.total_pages;

                }).catch(function (reason) {
                    /**
                     * Shows alert popup to inform the user, there are no available data
                     */
                    $scope.showAlertInvalidData($translate.instant('header_title_shows'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

                }).finally(function () {
                    /**
                     * Broadcast scroll event
                     */
                    $scope.$broadcast('scroll.infiniteScrollComplete');

                    /**
                     * Hides the loader
                     */
                    $ionicLoading.hide();
                });
            }else {
                /**
                 * Broadcast scroll event
                 */
                $scope.$broadcast('scroll.infiniteScrollComplete');
            }

        };

        /**
         * toggleShowMoreDescription
         * @param item
         */
        $scope.toggleShowMoreDescription = function (item) {
            if ($scope.isShowMoreDescriptionOpen(item)) {
                $ionicScrollDelegate.scrollTop(true);
                $scope.shownItem = null;
            } else {
                $scope.shownItem = item;
            }
        };
        $scope.isShowMoreDescriptionOpen = function (item) {
            return $scope.shownItem === item;
        };

        /**
         * Function to toggle the caret of the show description depending of the height of it.
         * @returns {boolean}
         */
        $scope.checkDescriptionBlockSize = function () {
            var descriptionBlock = document.getElementsByClassName("app-description-block");
            var currentWidth = $window.screen.width;
            if (currentWidth > 570) {
                return (descriptionBlock[0].clientHeight <= 70);
            } else {
                return false;
            }
        };
    });
