/**
 * Created by mivranis on 19/08/2016.
 */
angular.module('gr.ant1.app')
/**
 * Search Results Controller
 */
    .controller('SearchController', function (Shows, $log, $window, $rootScope, $scope, $stateParams, $ionicHistory, $translate, $timeout, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate, jsonValidator, InstantiatorService, CacheFactory, BASE_URL, SERVICE_SHOWS) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Search");

        /**
         * On Enter view event
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            $rootScope.checkCache(states, scopes, 'showsCache', BASE_URL.API_URL + SERVICE_SHOWS.SEARCH_RESULTS, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Search");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory using cue promise
         */
        $scope.updateViewData = function () {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Sets the search query data to scope
             */
            $scope.searchTitle = $translate.instant('header_title_search_' + $rootScope.searchCaller);

            /**
             * Fetch view data results from factory
             */
            Shows.getSearchResults($rootScope.searchQuery.search.text, $rootScope.searchCaller).then(function (result) {
                /**
                 * Sets search results scope object
                 */
                if (result.data.error === 1) {
                    $scope.searchResults = [];
                } else {
                    $scope.searchResults = result.data.data;
                }

            }).catch(function (reason) {
                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_shows'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();

                /**
                 * Remove the previous view from the history completely
                 */
                if ($ionicHistory.currentView().stateName === "app.search") {
                    if ($ionicHistory.backView().stateName !== "app.shows" && $ionicHistory.backView().stateName !== "app.showsArchive") {
                        removeBackView();
                    }
                }
            });
        };

        /**
         * Remove the previous view from the history completely, including the
         * cached element and scope (if they exist).
         */
        function removeBackView() {
            var currentHistory = $ionicHistory.viewHistory().histories[$ionicHistory.currentHistoryId()];
            var currentCursor = currentHistory.cursor;
            var currentView = currentHistory.stack[currentCursor];
            var backView = currentHistory.stack[currentCursor - 1];
            var replacementView = currentHistory.stack[currentCursor - 2];
            /**
             * Fail if we do not have enough views in the history
             */
            if (!backView || !replacementView) {
            } else {
                /**
                 * Remove the old backView and the cached element/scope
                 */
                currentHistory.stack.splice(currentCursor - 1, 1);
                $ionicHistory.clearCache([backView.viewId]);
                /**
                 * Make the replacementView and currentView point to each other
                 * (bypass the old backView)
                 */
                currentView.backViewId = replacementView.viewId;
                currentView.index = currentView.index - 1;
                replacementView.forwardViewId = currentView.viewId;
                /**
                 * Update the cursor and sets new backView
                 */
                $ionicHistory.viewHistory().backView = replacementView;
                currentHistory.currentCursor += -1;
            }
        }

    });
