/**
 * Created by vanger on 12/08/2016.
 */
angular.module('gr.ant1.app')
    /**
     * Shows Archive Controller
     */
    .controller('ShowsArchiveController', function (ShowsArchive, ShowsArchiveSchema, $log, $window, $timeout, $rootScope, $scope, $stateParams, $ionicHistory, $translate, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate, jsonValidator, InstantiatorService, $q, CacheFactory, BASE_URL, SERVICE_ARCHIVE) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Shows Archive");

        /**
         * Sets the caller for the search
         * @type {string}
         */
        $rootScope.searchCaller = 'archive';
        $rootScope.clearSearch();

        /**
         * The id of the genre
         * @type {string}
         */
        var genre_id = '';
        /**
         * The current page of shows for the selected category
         * @type {number}
         */
        var page = 1;
        /**
         * The total pages of shows for the selected category
         * @type {number}
         */
        var total_pages = 0;
        /**
         * To check the page ends last or not
         * @type {boolean}
         */
        $scope.moreDataCanBeLoaded = true;

        /**
         * Declare and initialize items object to scope
         * @type {{}}
         */
        $scope.items = {};

        /**
         * Declare and initialize shows schema object to scope
         * @type {Array}
         */
        $scope.schema_shows = [];

        /**
         * Workaround for slide-box
         */
        $scope.onTouch = function () {
            $ionicSlideBoxDelegate.enableSlide(false);
        };
        $scope.onRelease = function () {
            $ionicSlideBoxDelegate.enableSlide(true);
        };
        $scope.nextSlide = function () {
            $ionicSlideBoxDelegate.next(500);
        };
        $scope.previousSlide = function () {
            $ionicSlideBoxDelegate.previous(500);
        };

        /**
         * On Enter view event
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            /**
             * Sets the caller for the search
             * @type {string}
             */
            $rootScope.searchCaller = 'archive';
            $rootScope.clearSearch();

            /**
             * Check cache for data
             */
            $rootScope.checkCache(states, scopes, 'showsArchiveCache', BASE_URL.API_URL + SERVICE_ARCHIVE.URL, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Shows Archive");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory
         */
        $scope.updateViewData = function () {
            /**
             * Resets the id of the genre
             * @type {string}
             */
            genre_id = '';
            /**
             * Resets the current page of shows for the selected category
             * @type {number}
             */
            page = 1;
            /**
             * Resets the total pages of shows for the selected category
             * @type {number}
             */
            total_pages = 0;
            /**
             * To check the page ends last or not
             * @type {boolean}
             */
            $scope.moreDataCanBeLoaded = true;

            /**
             * Initialize items object to scope
             * @type {{}}
             */
            $scope.items = {};

            /**
             * Initialize tabs array to scope
             * @type {Array}
             */
            $scope.items.tabs = [];

            /**
             * Initialize content tab array to scope
             * @type {Array}
             */
            $scope.items.content = [];
            $scope.items.contentItems = [];

            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Promises array
             * @type {{schema_genres: *, schema_shows: *, genres: *}}
             */
            var promises = {
                /**
                 * Gets the genres schema object
                 */
                schema_genres: ShowsArchiveSchema.getShowGenres().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the shows schema object
                 */
                schema_shows: ShowsArchiveSchema.getShowsByGenre().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the genres
                 */
                genres: ShowsArchive.getShowGenres().then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return [];
                })
            };

            /**
             * Construct all data for genres and shows
             */
            $q.all(promises).then(function (result) {
                /**
                 * Sets the schema shows array
                 * @type {any}
                 */
                $scope.schema_shows = angular.copy(result.schema_shows);

                /**
                 * Sets scope tabs array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Object}
                 */
                $scope.items.tabs = angular.merge([], angular.copy(result.schema_genres), angular.copy(result.genres));

                /**
                 * Adds the category tab 'ALL' to categories array
                 * @type {{ContentId: string, Name: string}}
                 */
                $scope.items.tabs.unshift({ContentId: '', Name: 'ΟΛΑ', ShowOrder: -1});

                /**
                 * Enables sliding in slidebox
                 */
                $ionicSlideBoxDelegate.enableSlide(true);

                /**
                 * Sets the first genre shows to scope for use in view
                 */
                if ($scope.items.tabs.length > 0) {
                    /**
                     * Sets the id of the genre
                     */
                    genre_id = $scope.items.tabs[$ionicSlideBoxDelegate.currentIndex()].ContentId;
                    getShowsByGenreId(genre_id);
                }

            }).catch(function (response) {
                /**
                 * Declare and initialize items object to scope
                 * @type {{}}
                 */
                $scope.items = {};

                /**
                 * Declare and initialize tabs array to scope
                 * @type {Array}
                 */
                $scope.items.tabs = [];

                /**
                 * Declare and initialize content tab array to scope
                 * @type {Array}
                 */
                $scope.items.content = [];
                $scope.items.contentItems = [];

                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_shows'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };

        /**
         * Sets the tab content data by ContentId when slide changed
         * @param index
         */
        $scope.slideHasChanged = function (index) {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Scroll to top the list
             */
            $ionicScrollDelegate.scrollTop(true);

            /**
             * Sets id of the genre
             */
            genre_id = $scope.items.tabs[index].ContentId;

            /**
             * Resets the current page of shows for the selected category
             * @type {number}
             */
            page = 1;

            /**
             * Resets the total pages of shows for the selected category
             * @type {number}
             */
            total_pages = 0;

            /**
             * Declare and initialize content tab array to scope
             * @type {Array}
             */
            $scope.items.content = [];
            $scope.items.contentItems = [];

            /**
             * Request shows by genre id
             */
            getShowsByGenreId(genre_id);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackEvent("Shows Archive Category", $scope.items.tabs[index].Name, $scope.items.tabs[index].ContentId);
        };

        /**
         * Update slider view
         */
        $scope.updateSlider = function () {
            /**
             * Updates view
             */
            $ionicSlideBoxDelegate.enableSlide(true);
            $ionicScrollDelegate.scrollTop();
            $ionicSlideBoxDelegate.update();
            $timeout(function () {
                $ionicSlideBoxDelegate.enableSlide(true);
                $ionicScrollDelegate.scrollTop();
                $ionicSlideBoxDelegate.update();
            }, 500);
        };

        /**
         * Returns shows by genreId
         * @param genreId
         */
        function getShowsByGenreId(genreId) {
            /**
             * Gets the shows by genre
             */
            ShowsArchive.getShowsByGenre(genreId, page).then(function (result) {
                /**
                 * Sets shows by selected genre
                 * Merge schema objects with requested objects for always right structure
                 * @type {Object}
                 */
                var items = angular.copy(result.data.data);
                var len = items.length;
                if (len > 0) $scope.items.content = angular.merge([], $scope.schema_shows, items);

                /**
                 * Add data items to scope array
                 * @type {number}
                 */
                for (var i = 0; i < len; i++) {
                    $scope.items.contentItems.push($scope.items.content[i]);
                }

                /**
                 * Resets the total pages of shows for the selected category
                 */
                total_pages = result.data.total_pages;

                /**
                 * Resets the flag
                 */
                $scope.moreDataCanBeLoaded = true;

                /**
                 * Check if you have reached at bottom
                 */
                if (page >= total_pages) $scope.moreDataCanBeLoaded = false;

                /**
                 * Increment the current page
                 */
                page++;

            }).catch(function (reason) {
                /**
                 * Declare and initialize content tab array to scope
                 * @type {Array}
                 */
                $scope.items.content = angular.copy($scope.schema_shows);
                $scope.items.contentItems = angular.copy($scope.schema_shows);

                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_archive'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Broadcast scroll event
                 */
                $scope.$broadcast('scroll.infiniteScrollComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        }

        /**
         * Load more for infinite-scroll with paging request
         */
        $scope.loadMore = function () {
            if (page <= total_pages) {
                getShowsByGenreId(genre_id);
            } else {
                /**
                 * Broadcast scroll event
                 */
                $scope.$broadcast('scroll.infiniteScrollComplete');
            }
        };

        /**
         * Refactor Show Genres
         *
         * @param genres
         * @param shows
         * @returns {Array}
         */
        function refactorGenres(genres, shows) {
            var data = [];
            angular.forEach(genres, function (category, key) {
                var keepGoing = true;
                angular.forEach(shows, function (show, key) {
                    if (keepGoing) {
                        if (category.Name === show.Genre) {
                            data.push(category);
                            keepGoing = false;
                        }
                    }
                });
            });
            return data;
        }
    });
