/**
 * Created by vanger on 12/08/2016.
 */
angular.module('gr.ant1.app')
    /**
     * Shows Controller
     */
    .controller('ShowsController', function (Shows, ShowsSchema, $log, $window, $timeout, $rootScope, $scope, $stateParams, $ionicHistory, $translate, $ionicLoading, $ionicSlideBoxDelegate, $ionicScrollDelegate, jsonValidator, InstantiatorService, $q, CacheFactory, BASE_URL, SERVICE_SHOWS) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Shows");

        /**
         * Sets the caller for the search
         * @type {string}
         */
        $rootScope.searchCaller = 'shows';
        $rootScope.clearSearch();

        /**
         * The id of the category
         * @type {string}
         */
        var category_id = '';
        /**
         * The current page of shows for the selected category
         * @type {number}
         */
        var page = 1;
        /**
         * The total pages of shows for the selected category
         * @type {number}
         */
        var total_pages = 0;
        /**
         * To check the page ends last or not
         * @type {boolean}
         */
        $scope.moreDataCanBeLoaded = true;

        /**
         * Declare and initialize shows schema object to scope
         * @type {Array}
         */
        $scope.schema_shows = [];

        /**
         * Declare and initialize categories array to scope
         * @type {Array}
         */
        $scope.categories = [];

        /**
         * Declare and initialize shows array to scope
         * @type {Array}
         */
        $scope.shows = [];

        /**
         * Declare and initialize show items array to scope
         * @type {Array}
         */
        $scope.showsItems = [];

        /**
         * Workaround for slide-box
         */
        $scope.onTouch = function () {
            $ionicSlideBoxDelegate.enableSlide(false);
        };
        $scope.onRelease = function () {
            $ionicSlideBoxDelegate.enableSlide(true);
        };
        $scope.nextSlide = function () {
            $ionicSlideBoxDelegate.next(500);
        };
        $scope.previousSlide = function () {
            $ionicSlideBoxDelegate.previous(500);
        };

        /**
         * On Enter view event
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            /**
             * Sets the caller for the search
             * @type {string}
             */
            $rootScope.searchCaller = 'shows';
            $rootScope.clearSearch();

            /**
             * Check cache for data
             */
            $rootScope.checkCache(states, scopes, 'showsCache', BASE_URL.API_URL + SERVICE_SHOWS.URL, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Shows");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory
         */
        $scope.updateViewData = function () {
            /**
             * Resets the id of the category
             * @type {string}
             */
            category_id = '';
            /**
             * Resets the current page of shows for the selected category
             * @type {number}
             */
            page = 1;
            /**
             * Resets the total pages of shows for the selected category
             * @type {number}
             */
            total_pages = 0;
            /**
             * To check the page ends last or not
             * @type {boolean}
             */
            $scope.moreDataCanBeLoaded = true;

            /**
             * Initialize shows schema object to scope
             * @type {Array}
             */
            $scope.schema_shows = [];

            /**
             * Initialize categories array to scope
             * @type {Array}
             */
            $scope.categories = [];

            /**
             * Initialize shows array to scope
             * @type {Array}
             */
            $scope.shows = [];

            /**
             * Initialize show items array to scope
             * @type {Array}
             */
            $scope.showsItems = [];

            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Promises array
             * @type {{schema_categories: *, schema_shows: *, categories: *}}
             */
            var promises = {
                /**
                 * Gets the categories schema object
                 */
                schema_categories: ShowsSchema.getCategories().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the shows schema object
                 */
                schema_shows: ShowsSchema.getShowsByMenuCategory().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets shows categories
                 */
                categories: Shows.getCategories().then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return [];
                })
            };

            /**
             * Construct all data for categories and shows
             */
            $q.all(promises).then(function (result) {
                /**
                 * Sets the schema shows array
                 * @type {any}
                 */
                $scope.schema_shows = angular.copy(result.schema_shows);

                /**
                 * Sets scope categories array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                $scope.categories = angular.merge([], angular.copy(result.schema_categories), angular.copy(result.categories));

                /**
                 * Adds the category tab 'ALL' to categories array
                 * @type {{ContentId: string, Title: string, ShowOrder: number, Color: string}}
                 */
                $scope.categories.unshift({ContentId: '', Title: 'ΟΛΑ', ShowOrder: -1, Color: ''});

                /**
                 * Enables sliding in slidebox
                 */
                $ionicSlideBoxDelegate.enableSlide(true);

                /**
                 * Sets the first category shows to scope for use in view
                 */
                if ($scope.categories.length > 0) {
                    /**
                     * Sets the id of the category
                     */
                    category_id = $scope.categories[$ionicSlideBoxDelegate.currentIndex()].ContentId;
                    getShowsByMenuCategory(category_id);
                }

            }).catch(function (reason) {
                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_shows'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };

        /**
         * Slide change event handler
         * @param index
         */
        $scope.slideHasChanged = function (index) {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Scroll to top the list
             */
            $ionicScrollDelegate.scrollTop(true);

            /**
             * Sets the id of the category
             */
            category_id = $scope.categories[index].ContentId;

            /**
             * Resets the current page of shows for the selected category
             * @type {number}
             */
            page = 1;

            /**
             * Resets the total pages of shows for the selected category
             * @type {number}
             */
            total_pages = 0;

            /**
             * Initialize shows array to scope
             * @type {Array}
             */
            $scope.shows = [];
            /**
             * Initialize shows items array to scope
             * @type {Array}
             */
            $scope.showsItems = [];

            /**
             * Request shows by category id
             */
            getShowsByMenuCategory(category_id);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackEvent("Shows Category", $scope.categories[index].Title, $scope.categories[index].ContentId);
        };

        /**
         * Update slider view
         */
        $scope.updateSlider = function () {
            /**
             * Updates view
             */
            $ionicSlideBoxDelegate.enableSlide(true);
            $ionicScrollDelegate.scrollTop(true);
            $ionicSlideBoxDelegate.update();
            $timeout(function () {
                $ionicSlideBoxDelegate.enableSlide(true);
                $ionicScrollDelegate.scrollTop(true);
                $ionicSlideBoxDelegate.update();
            }, 500);
        };

        /**
         * Returns shows by content id
         *
         * @param categoryId
         */
        function getShowsByMenuCategory(categoryId) {
            /**
             * Reset shows array
             * @type {Array}
             */
            $scope.shows = [];

            /**
             * Gets the shows by menu category
             */
            Shows.getShowsByMenuCategory(categoryId, page).then(function (result) {
                /**
                 * Sets scope shows array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                var items = angular.copy(result.data.data);
                var len = items.length;
                if (len > 0) $scope.shows = angular.merge([], $scope.schema_shows, items);

                /**
                 * Add data items to scope array
                 * @type {number}
                 */
                for (var i = 0; i < len; i++) {
                    $scope.showsItems.push($scope.shows[i]);
                }

                /**
                 * Resets the total pages of shows for the selected category
                 */
                total_pages = result.data.total_pages;

                /**
                 * Resets the flag
                 */
                $scope.moreDataCanBeLoaded = true;

                /**
                 * Check if you have reached at bottom
                 */
                if (page >= total_pages) $scope.moreDataCanBeLoaded = false;

                /**
                 * Increment the current page
                 */
                page++;

            }).catch(function (reason) {
                /**
                 * Declare and initialize shows array to scope
                 * @type {Array}
                 */
                $scope.shows = angular.copy($scope.schema_shows);
                $scope.showsItems = angular.copy($scope.schema_shows);

                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_shows'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Broadcast scroll event
                 */
                $scope.$broadcast('scroll.infiniteScrollComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        }

        /**
         * Load more with infinite-scroll
         */
        $scope.loadMore = function () {
            if (page <= total_pages) {
                getShowsByMenuCategory(category_id);
            } else {
                /**
                 * Broadcast scroll event
                 */
                $scope.$broadcast('scroll.infiniteScrollComplete');
            }
        };

        /**
         * Refactor Show Categories
         *
         * @param categories
         * @param shows
         * @returns {Array}
         */
        function refactorCategories(categories, shows) {
            var data = [];
            angular.forEach(categories, function (category, key) {
                var keepGoing = true;
                angular.forEach(shows, function (show, key) {
                    if (keepGoing) {
                        if (category.ContentId === show.Category) {
                            data.push(category);
                            keepGoing = false;
                        }
                    }
                });
            });
            return data;
        }
    });
