/**
 * Created by vanger on 12/08/2016.
 */
angular.module('gr.ant1.app')
    /**
     * Video Trailers Controller
     */
    .controller('VideoTrailersController', function (VideoTrailers, VideoTrailersSchema, $log, $window, $timeout, $rootScope, $scope, $stateParams, $ionicHistory, $translate, $ionicLoading, jsonValidator, InstantiatorService, $q, CacheFactory, BASE_URL, SERVICE_TRAILERS) {
        /**
         * Sets the page title for the history
         */
        $ionicHistory.currentTitle("Video Trailers");

        /**
         * Declare and initialize trailers array to scope
         * @type {Array}
         */
        $scope.trailers = [];

        /**
         * On Enter view event
         */
        $scope.$on('$ionicView.enter', function (states, scopes) {
            $rootScope.checkCache(states, scopes, 'videoTrailersCache', BASE_URL.API_URL + SERVICE_TRAILERS.URL, $scope, CacheFactory);

            /**
             * Google Analytics Track View
             */
            if ($window.ga) $window.ga.trackView("Video Trailers");
        });

        /**
         * Refresh view data from 'scroll to refresh action'
         */
        $scope.doRefresh = function () {
            $scope.updateViewData();
        };

        /**
         * Fetch view data from factory
         */
        $scope.updateViewData = function () {
            /**
             * Shows the loader
             */
            $ionicLoading.show();

            /**
             * Promises array
             *
             * @type {{schema_trailers: *, trailers: *}}
             */
            var promises = {
                /**
                 * Gets the trailers schema object
                 */
                schema_trailers: VideoTrailersSchema.getTrailers().then(function (value) {
                    return InstantiatorService.instantiate(value.data).data;
                }, function (reason) {
                    return [];
                }),
                /**
                 * Gets the trailers
                 */
                trailers: VideoTrailers.getTrailers().then(function (value) {
                    return value.data.data;
                }, function (reason) {
                    return [];
                })
            };

            /**
             * Construct all data for categories and shows
             */
            $q.all(promises).then(function (result) {
                /**
                 * Sets scope trailers array
                 * Merge schema objects with requested objects for always right structure
                 * @type {Array}
                 */
                $scope.trailers = angular.merge([], result.schema_trailers, result.trailers);

            }).catch(function (reason) {
                /**
                 * Shows alert popup to inform the user, there are no available data
                 */
                $scope.showAlertInvalidData($translate.instant('header_title_trailers'), $translate.instant('alert_no_data'), '', $scope.doRefresh);

            }).finally(function () {
                /**
                 * Stop the ion-refresher from spinning
                 */
                $scope.$broadcast('scroll.refreshComplete');

                /**
                 * Hides the loader
                 */
                $ionicLoading.hide();
            });
        };
    });
