angular
    /**
     *  Application Module Name
     */
    .module('gr.ant1.app')

    /*
     |-----------------------------------------------------------------------------------
     | Menu Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getMenu: Returns the menu data
     |
     */
    .factory('Menu', function ($http, CacheFactory, BASE_URL, SERVICE_MENU) {

        CacheFactory('menuCache', {
            maxAge: SERVICE_MENU.CACHE_MAX_AGE
        });

        return {
            getMenu: function () {
                return $http.jsonp(BASE_URL.API_URL + SERVICE_MENU.URL, {cache: CacheFactory.get('menuCache')});
            },
            getMenuLocal: function () {
                return $http.get(SERVICE_MENU.LOCAL_URL, {cache: CacheFactory.get('menuCache')});
            }
        }
    })

    /*
     |-----------------------------------------------------------------------------------
     | Homepage Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getCarousel:
     |           Title		    : ο τίτλος του carousel
     |           Subtitle	    : o υπότιτλος του carousel
     |           CoverImage	    : to absolute url της εικόνας σε 768
     |           ScrSet		    : το url με το image set του carousel slider
     |           TypeOfUrl      : 0,1,2,3 (0 – default τιμή, 1 – ShowId, 2 – VideoUrl, 3 – External Url)
     |           Link		    : ShowId / Video Url / External Url
     |           Url			: το url του video στο site. Εφόσον δεν υπάρχει τιμή δε θα πρέπει
     |                            να δείχνουμε το share στον player γιατί υπάρχει περίπτωση το video να μην έχει σελίδα
     |
     |      getShowLatestEpisodes:
     |           OwnerTitle		: Αυτός είναι ο τίτλος του επεισοδίου
     |           CoverImage		: το cover image του αρχείου
     |           MediaFileRef	: το URL του αρχείου
     |           SrcSet			: image source set
     |           Url			: το url του επεισοδίου στο site
     |
     |      getRecommended:
     |           IsShow			: true/false
     |           ShowId			: ο id της εκπομπής
     |           Title			: ο τίτλος της εκπομπής η του επεισοδίου
     |           Image			: το image της εκπομπής η του επεισοδίου
     |           MediaFileRef	: το URL του αρχείου/ κενό άμα είναι show
     |           SrcSet			: το srcset της εκπομπής η του επεισοδίου
     |           Url			: το url του επεισοδίου στο site / κενό άμα είναι show
     |
     */
    .factory('Homepage', function ($http, CacheFactory, BASE_URL, SERVICE_HOMEPAGE) {
        var url = BASE_URL.API_URL;

        CacheFactory('homepageCache', {
            maxAge: SERVICE_HOMEPAGE.CACHE_MAX_AGE
        });

        return {
            getCarousel: function () {
                return $http.jsonp(url + SERVICE_HOMEPAGE.CAROUSEL, {cache: CacheFactory.get('homepageCache')});
            },
            getLatestEpisodes: function () {
                return $http.jsonp(url + SERVICE_HOMEPAGE.LATEST_EPISODES, {cache: CacheFactory.get('homepageCache')});
            },
            getRecommended: function () {
                return $http.jsonp(url + SERVICE_HOMEPAGE.RECOMMENDED, {cache: CacheFactory.get('homepageCache')});
            }
        }
    })

    /*
     |-----------------------------------------------------------------------------------
     | Schedule Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getTabs: Returns the weekly schedule tabs data
     |          meta:
     |              WeekDate	    : ex.: "08/11"
     |              WeekDay	        : ex.: "Πέμπτη"
     |
     |          items:
     |              StartDate	    : ποτέ αρχίζει η εκπομπή
     |              EndDate		    : ποτέ τελειώνει η εκπομπή
     |              ShowTitle	    : ο τίτλος της εκπομπής
     |              ShowId		    : το id της εκπομπής
     |              Color		    : το χρώμα της κατηγορίας
     |
     */
    .factory('Schedule', function ($http, CacheFactory, BASE_URL, SERVICE_SCHEDULE) {
        var url = BASE_URL.API_URL + SERVICE_SCHEDULE.URL;

        CacheFactory('scheduleCache', {
            maxAge: SERVICE_SCHEDULE.CACHE_MAX_AGE
        });

        return {
            getTabs: function () {
                return $http.jsonp(url, {cache: CacheFactory.get('scheduleCache')});
            }
        }
    })

    /*
     |-----------------------------------------------------------------------------------
     | Shows Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getCategories : Returns the categories of the shows
     |          ContentId	    : το id της κατηγορίας
     |          Title		    : ο τίτλος
     |          ShowOrder	    : η σειρά εμφάνισης
     |          Color		    : το χρώμα σε hex
     |
     |      getShowsByMenuCategory: Returns the shows per show category, if the parameter 'menu'
     |                              is missing the returns all shows for the page
     |          Menu		    : ο τίτλος του menu
     |          Title		    : ο τίτλος του show
     |          Order		    : σειρά εμφάνισης
     |          CoverImage	    : η εικόνα της εκπομπής
     |          ShowId		    : το id του show
     |          SrcSet			: το image source set
     |          Category	    : η κατηγορία που ανήκει το show
     |
     |      getEpisodesForShow : Returns all the episodes for a specific show
     |          OwnerTitle		: Αυτός είναι ο τίτλος του επεισοδίου
     |          CoverImage		: το cover image του αρχείου
     |          MediaFileRef	: το URL του αρχείου
     |          SrcSet			: το image source set
     |          Url			    : το url του επεισοδίου στο site
     |
     |      getShowDetailsById : Returns the shows details of a specific show
     |          Title		    : ο τίτλος της εκπομπής
     |          HeaderImage     : το λογότυπο της εκπομπής για σελίδα εκπομπών
     |          Description	    : η περιγραφή της εκπομπής
     |          SrcSet		    : το image source set
     |
     */
    .factory('Shows', function ($http, CacheFactory, BASE_URL, SERVICE_SHOWS) {
        var url = BASE_URL.API_URL;

        CacheFactory('showsCache', {
            maxAge: SERVICE_SHOWS.CACHE_MAX_AGE
        });

        return {
            getCategories: function () {
                return $http.jsonp(url + SERVICE_SHOWS.CATEGORIES, {cache: CacheFactory.get('showsCache')});
            },
            getShowsByMenuCategory: function (contentId, page) {
                return $http.jsonp(url + SERVICE_SHOWS.BY_CATEGORY, {
                    cache: CacheFactory.get('showsCache'),
                    params: {
                        menu: contentId
                        , page: page
                    }
                });
            },
            getEpisodesForShow: function (showId, page) {
                return $http.jsonp(url + SERVICE_SHOWS.SHOW_EPISODES, {
                    cache: CacheFactory.get('showsCache'),
                    params: {
                        show: showId
                        , page: page
                    }
                });
            },
            getShowDetailsById: function (showId) {
                return $http.jsonp(url + SERVICE_SHOWS.SHOW_DETAILS, {
                    cache: CacheFactory.get('showsCache'),
                    params: {
                        show: showId
                    }
                });
            },
            getSearchResults: function (queryData, isLive) {
                var params = {search: queryData};
                if (isLive == 'shows') params.active = 1;

                return $http.jsonp(url + SERVICE_SHOWS.SEARCH_RESULTS, {
                    cache: false,
                    params: params
                });
            }
        }
    })

    /*
     |-----------------------------------------------------------------------------------
     | Shows Archive Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getShowGenres : Returns the archive categories
     |          ContentId	: το id της κατηγορίας
     |          Name		: το όνομα της κατηγορίας
     |
     |      getShowsByGenre : Returns the shows per archive category, if the parameter 'genre'
     |                              is missing the returns all shows for the page
     |          ShowId	    : το id της εκπομπής
     |          Title		: ο τίτλος της εκπομπής
     |          CoverImage	: το λογότυπο της εκπομπής
     |          SrcSet		: image source set
     |          Genre		: περιγραφή του είδος της εκπομπής
     |
     */
    .factory('ShowsArchive', function ($http, CacheFactory, BASE_URL, SERVICE_ARCHIVE) {
        var url = BASE_URL.API_URL;

        CacheFactory('showsArchiveCache', {
            maxAge: SERVICE_ARCHIVE.CACHE_MAX_AGE
        });

        return {
            getShowGenres: function () {
                return $http.jsonp(url + SERVICE_ARCHIVE.GENRES, {cache: CacheFactory.get('showsArchiveCache')});
            },
            getShowsByGenre: function (genreId, page) {
                return $http.jsonp(url + SERVICE_ARCHIVE.SHOWS_BY_GENRE, {
                    cache: CacheFactory.get('showsArchiveCache'),
                    params: {
                        genre: genreId
                        , page: page
                    }
                });
            }
        }
    })

    /*
     |-----------------------------------------------------------------------------------
     | Video Trailers Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getVideoTrailers: Returns the video trailers
     |          OwnerTitle		: Αυτός είναι ο τίτλος του κόμβου (episode, season) στον οποίο ανήκει το συγκεκριμένο medium
     |          CoverImage		: το cover image του αρχείου
     |          MediaFileRef	: το URL του αρχείου
     |          SrcSet			: image source set
     |          Url			    : το url του trailer στο site
     |
     */
    .factory('VideoTrailers', function ($http, CacheFactory, BASE_URL, SERVICE_TRAILERS) {
        var url = BASE_URL.API_URL + SERVICE_TRAILERS.URL;

        CacheFactory('videoTrailersCache', {
            maxAge: SERVICE_TRAILERS.CACHE_MAX_AGE
        });

        return {
            getTrailers: function () {
                return $http.jsonp(url, {cache: CacheFactory.get('videoTrailersCache')});
            }
        }
    })

    /*
     |-----------------------------------------------------------------------------------
     | Live Stream Factory
     |-----------------------------------------------------------------------------------
     | - FUNCTIONS
     |
     |      getLiveStream       : Returns the live stream permission
     |
     */
    .factory('LiveStream', function ($http, CacheFactory, BASE_URL, SERVICE_VIDEO_STREAM) {
        var url = BASE_URL.API_URL + SERVICE_VIDEO_STREAM.URL;

        return {
            getLiveStream: function () {
                return $http.jsonp(url, {cache:false});
            }
        }
    });