angular

    /**
     * Application Module Name
     */
    .module('gr.ant1.app')


    /**
     * Connectivity Monitor Factory
     */
    .factory('ConnectivityMonitor', function($rootScope, $cordovaNetwork){
        /**
         * Set the online check flag to true (initialize)
         * @type {boolean}
         */
        $rootScope.isOnlineFlag = true;

        return {
            /**
             * isOnline
             * Checks if the device is Online
             *
             * @returns {*}
             */
            isOnline: function(){
                if(ionic.Platform.isWebView()){
                    return $cordovaNetwork.isOnline();
                }else {
                    return navigator.onLine;
                }
            },

            /**
             * isOffline
             * Checks if the device is Offline
             *
             * @returns {*}
             */
            isOffline: function(){
                if(ionic.Platform.isWebView()){
                    return !$cordovaNetwork.isOnline();
                }else {
                    return !navigator.onLine;
                }
            },

            /**
             * startWatching
             *
             * Adds events to monitor network connectivity changes
             */
            startWatching: function(){
                if(!$rootScope.ConnectivityMonitorIsWatching) {
                    if (ionic.Platform.isWebView()) {
                        $rootScope.$on('$cordovaNetwork:online', $rootScope.isOnline);
                        $rootScope.$on('$cordovaNetwork:offline', $rootScope.isOffline);
                    } else {
                        window.addEventListener("online", $rootScope.isOnline, false);
                        window.addEventListener("offline", $rootScope.isOffline, false);
                    }
                    $rootScope.ConnectivityMonitorIsWatching = true;
                }
            },

            /**
             * stopWatching
             *
             * Removes events to monitor network connectivity changes
             */
            stopWatching: function(){
                if($rootScope.ConnectivityMonitorIsWatching) {
                    if(ionic.Platform.isWebView()){
                        $rootScope.$on('$cordovaNetwork:online', null);
                        $rootScope.$on('$cordovaNetwork:offline', null);
                    }else {
                        window.removeEventListener("online", $rootScope.isOnline);
                        window.removeEventListener("online", $rootScope.isOffline);
                    }
                    $rootScope.ConnectivityMonitorIsWatching = false;
                }
            },

            /**
             * isWatching
             *
             * Checks if @startWatching has been initiated
             */
            isWatching: function(){
                return $rootScope.ConnectivityMonitorIsWatching;
            }
        }
    });