angular
    /**
     * Application Module Name
     */
    .module('gr.ant1.app')
    /**
     * Notify Factory
     */
    .factory('NotifyFactory', function ($log, $window, NOTIFY_CONFIG) {

        /**
         * Send to selected Notify Service
         */
        var captureException = function (msg, obj) {
            switch (NOTIFY_CONFIG.ACTIVE) {
                case NOTIFY_CONFIG.NAME_SENTRY:
                    sendSentry(msg, obj);
                    break;
                case NOTIFY_CONFIG.NAME_ATATUS:
                    sendAtatus(msg, obj);
                    break;
            }
        };

        /**
         * Send to selected Notify Service
         */
        var captureGlobalException = function (exception) {
            switch (NOTIFY_CONFIG.ACTIVE) {
                case NOTIFY_CONFIG.NAME_SENTRY:
                    sendGlobalSentry(exception);
                    break;
                case NOTIFY_CONFIG.NAME_ATATUS:
                    sendGlobalAtatus(exception);
                    break;
            }
        };

        /**
         * Sentry :: Send notify with Sentry Service
         */
        var sendSentry = function (msg, obj) {
            Raven.captureMessage(msg, {extra: obj});
        };

        /**
         * Sentry :: Send global notify with Sentry Service
         */
        var sendGlobalSentry = function (exception) {
            Raven.captureMessage(exception);
        };

        /**
         * Atatus :: Send global notify with Atatus Service
         */
        var sendGlobalAtatus = function (exception) {
            if ($window.atatus) {
                $window.atatus.notify(exception);
            }
        };

        /**
         * Atatus :: Send notify with Atatus Service
         */
        var sendAtatus = function (msg, obj) {
            if ($window.atatus) {
                $window.atatus.notify(msg, obj);
            }
        };
        /**
         * Atatus :: Beginning a Transaction
         *
         * The beginTransaction method denotes the start of a transaction.
         * At this point, Atatus begins timing the duration of the transaction.
         *
         */
        var beginTransactionAtatus = function (key_transaction_name) {
            switch (NOTIFY_CONFIG.ACTIVE) {
                case NOTIFY_CONFIG.NAME_SENTRY:
                    break;
                case NOTIFY_CONFIG.NAME_ATATUS:
                    if ($window.atatus) {
                        $window.atatus.beginTransaction(key_transaction_name);
                    }
                    break;
            }
        };
        /**
         * Atatus :: Ending a Transaction
         *
         * The endTransaction method tells Atatus that the given transaction has successfully completed.
         * At this point, the transaction time is recorded, and the transaction will be reported to Atatus.
         *
         */
        var endTransactionAtatus = function (key_transaction_name) {
            switch (NOTIFY_CONFIG.ACTIVE) {
                case NOTIFY_CONFIG.NAME_SENTRY:
                    break;
                case NOTIFY_CONFIG.NAME_ATATUS:
                    if ($window.atatus) {
                        $window.atatus.endTransaction(key_transaction_name);
                    }
                    break;
            }
        };
        /**
         * Atatus :: Failing a Transaction
         *
         * The failTransaction method tells Atatus that the given transaction has failed. At this point,
         * the transaction time is recorded, and the transaction will be reported to Atatus.
         *
         */
        var failTransactionAtatus = function (key_transaction_name) {
            switch (NOTIFY_CONFIG.ACTIVE) {
                case NOTIFY_CONFIG.NAME_SENTRY:
                    break;
                case NOTIFY_CONFIG.NAME_ATATUS:
                    if ($window.atatus) {
                        $window.atatus.failTransaction(key_transaction_name);
                    }
                    break;
            }
        };
        /**
         * Atatus :: Failing a Transaction
         *
         * The cancelTransaction method tells Atatus that the given transaction has canceled and
         * will not be reported to Atatus. Canceled transactions are ignored.
         *
         * You have to call cancelTransaction instead of endTransaction or failTransaction to cancel the transaction.
         */
        var cancelTransactionAtatus = function (key_transaction_name) {
            switch (NOTIFY_CONFIG.ACTIVE) {
                case NOTIFY_CONFIG.NAME_SENTRY:
                    break;
                case NOTIFY_CONFIG.NAME_ATATUS:
                    if ($window.atatus) {
                        $window.atatus.cancelTransaction(key_transaction_name);
                    }
                    break;
            }
        };

        /**
         * Return functions
         */
        return {
            captureException: captureException,
            captureGlobalException: captureGlobalException,
            beginTransactionAtatus: beginTransactionAtatus,
            endTransactionAtatus: endTransactionAtatus,
            failTransactionAtatus: failTransactionAtatus,
            cancelTransactionAtatus: cancelTransactionAtatus
        };
    });