/**
 * Created by mivranis on 25/8/2016.
 */

/**
 * directive to replace the main image given the dimensions of the screen
 */
angular.module('gr.ant1.app')
    .directive('mediaSrcApp', ['$window', function ($window) {
        function link(scope, element, attrs) {
            var imageSelected = "";
            var classArray = attrs.class.split(' ');

            scope.$watch('mediaSrcApp', function (mediaSrcApp) {
                scope.width = $window.innerWidth;
                if (mediaSrcApp !== undefined) {
                    imageSelected = selectImage(scope.width, mediaSrcApp, classArray);
                    element.attr("src", imageSelected);
                }
            });

            angular.element($window).bind('resize', function () {
                scope.width = $window.innerWidth;
                imageSelected = selectImage(scope.width, scope.mediaSrcApp, classArray);
                element.attr("src", imageSelected);
                scope.$digest();
            });
        }

        /**
         * Function which is called upon window resize / window rotation.
         * It returns the desired image size.
         * Angular Attempt to reqrite the to-be-introduced HTML5 attribute <picture> and the responsive <source> inside
         *
         * @param width
         * @param imageObject
         * @param classArray
         * @returns {string}
         */
        function selectImage(width, imageObject, classArray) {
            var imgSelected = "";

            if (width >= 320 && width <= 570) {
                if (imageObject != undefined && imageObject.media570 != undefined) {
                    imgSelected = imageObject.media570;
                } else {
                    imgSelected = getFallbackImage('xs', classArray);
                }
            } else if (width > 570 && width <= 768) {
                if (imageObject != undefined && imageObject.media768 != undefined) {
                    imgSelected = imageObject.media768;
                } else {
                    imgSelected = getFallbackImage('sm', classArray);
                }
            } else {
                if (imageObject != undefined && imageObject.media1024 != undefined) {
                    imgSelected = imageObject.media1024;
                } else {
                    imgSelected = getFallbackImage('md', classArray);
                }
            }
            return imgSelected;
        }

        /**
         * This function calculates and serves the respective fallback images in case the api images are broken.
         *
         * @param size
         * @param className
         * @returns {string}
         */
        function getFallbackImage(size, className) {
            var imgURL = 'img/default/';
            if (className.indexOf('app-home-carousel') >= 0) {
                switch (size) {
                    case 'xs':
                        return imgURL + 'noimage640x360.png';
                        break;
                    case 'sm':
                        return imgURL + 'noimage768x400.png';
                        break;
                    case 'md':
                        return imgURL + 'noimage1024x310.png';
                        break;
                }
            } else if (className.indexOf('app-home-episodes') >= 0) {
                switch (size) {
                    case 'xs':
                        return imgURL + 'noimage640x360.png';
                        break;
                    case 'sm':
                        return imgURL + 'noimage350x195.png';
                        break;
                    case 'md':
                        return imgURL + 'noimage310x174.png';
                        break;
                }
            } else if (className.indexOf('app-shows-trailers') >= 0) {
                switch (size) {
                    case 'xs':
                        return imgURL + 'noimage640x360.png';
                        break;
                    case 'sm':
                        return imgURL + 'noimage768x400.png';
                        break;
                    case 'md':
                        return imgURL + 'noimage1024x400.png';
                        break;
                }
            } else if (className.indexOf('app-shows-details-cover') >= 0) {
                switch (size) {
                    case 'xs':
                        return imgURL + 'noimage640x360.png';
                        break;
                    case 'sm':
                        return imgURL + 'noimage768x400.png';
                        break;
                    case 'md':
                        return imgURL + 'noimage1024x400.png';
                        break;
                }
            } else if (className.indexOf('app-shows-details-episodes') >= 0) {
                switch (size) {
                    case 'xs':
                        return imgURL + 'noimage640x360.png';
                        break;
                    case 'sm':
                        return imgURL + 'noimage350x195.png';
                        break;
                    case 'md':
                        return imgURL + 'noimage310x174.png';
                        break;
                }
            }
        }

        return {
            link: link,
            scope: {
                mediaSrcApp: '='
            }
        };
    }]);