app.run(function ($rootScope, $state, $translate, $log, $window, $document, $ionicPopup, $cordovaSocialSharing, ConnectivityMonitor, $ionicScrollDelegate) {
    /**
     * Global function to check if the data from a view are already cached or expired and
     * update them accordingly.
     *
     * @param states
     * @param scopes
     * @param cacheRepository
     * @param url
     * @param $scope
     * @param CacheFactory
     */
    $rootScope.checkCache = function (states, scopes, cacheRepository, url, $scope, CacheFactory) {
        if (!angular.isUndefined(CacheFactory.get(cacheRepository).info(url))) {
            if (CacheFactory.get(cacheRepository).info(url).isExpired) {
                $scope.updateViewData();
            } else {
                if (!scopes.fromCache) {
                    $scope.updateViewData();
                }
            }
        } else {
            $scope.updateViewData();
        }
    };

    /**
     * Global function that listens network change events,
     * and is fired when the device is Online
     *
     * @param e
     * @param networkState
     */
    $rootScope.isOnline = function (e, networkState) {
        $rootScope.isOnlineFlag = true;
        if ($rootScope.isOfflinePopup) $rootScope.isOfflinePopup.close();
    };
    /**
     * Global function that listens network change events,
     * and is fired when the device is Offline
     *
     * @param e
     * @param networkState
     */
    $rootScope.isOffline = function (e, networkState) {
        $rootScope.isOnlineFlag = false;
        var showAlert = function () {
            $rootScope.isOfflinePopup = $ionicPopup.alert({
                buttons: [],
                title: 'Application Offline',
                subTitle: 'Please check your internet connection and try again!'
            });

            $rootScope.isOfflinePopup.then(function (res) {
                if (res && ConnectivityMonitor.isOffline()) showAlert();
            });
        };
        showAlert();
    };

    /**
     * Show alert popup if before view gets invalid or undefined data
     * Reference: http://ionicframework.com/docs/api/service/$ionicPopup/
     *
     * @param title     The title of the popup
     * @param subTitle  The subtitle of the popup
     * @param template  The html template below the subtitle
     * @param scope     The function that will be executed when OK button clicked
     */
    $rootScope.showAlertInvalidData = function (title, subTitle, template, scope) {
        $ionicPopup.alert({
            title: title,
            subTitle: subTitle,
            template: template,
            buttons: [
                {text: $translate.instant('alert_button_close')},
                {
                    text: '<b>' + $translate.instant('alert_button_refresh') + '</b>',
                    type: 'button-positive',
                    onTap: function (e) {
                        scope();
                    }
                }
            ]
        });
    };

    /**
     * Global function that listens for any navigation inside the app, and if offline
     * prevents user prom navigating.
     */
    $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {
        // Prevent user form navigating to another view if is Offline
        if (ConnectivityMonitor.isWatching()) {
            if (ConnectivityMonitor.isOffline()) event.preventDefault();
        }

        //

        if($rootScope.homeCarouselTimeoutID){
            clearTimeout($rootScope.homeCarouselTimeoutID);
        }

    });


    /**
     * shareAnywhere
     * Share images, text, messages via Facebook, Twitter, Email, SMS, WhatsApp, etc using this function.
     * Share via native share sheet
     *
     * Reference: http://ngcordova.com/docs/plugins/socialSharing/
     * Demo: https://github.com/petehouston/demo-ionic-social-sharing
     *
     * @param message
     * @param subject
     * @param image
     * @param link
     */
    $rootScope.shareAnywhere = function (message, subject, image, link) {
        $cordovaSocialSharing.share(message, subject, image, link).then(function (result) {
            /**
             * Social share success
             */
            if (result) {
                /**
                 * Google Analytics Track View
                 */
                if ($window.ga) $window.ga.trackEvent("Share", message, link);
            }

        }).catch(function (reason) {
            /**
             * Catch any share error
             */
        });
    };

    /**
     * genericBackButtonHandler
     * Function which controls the back button behaviour topping the default behavriours
     */
    $rootScope.genericBackButtonHandler = function () {
        if ($rootScope.playerInstance && ($rootScope.playerInstance.getState() == 'playing' || $rootScope.playerInstance.getState() == 'PLAYING')) {
            jwplayer('video-player').remove();
        }
    }

    var timer;

    $rootScope.checkScroll = function() {
        clearTimeout(timer);
        if(!$document[0].body.classList.contains('disable_clicks')) {
            $document[0].body.classList.add('disable_clicks')
        }
        timer = setTimeout(function(){
            $document[0].body.classList.remove('disable_clicks')
        },200);
    };

});

/**
 * Global Function set which opens Modal window using the video.html template.
 * The video.html has container for JW player which is initialised in here
 * Extra functions included: openVideoModal and closeModal
 *
 */
app.run(function ($ionicModal, $rootScope, JWPLAYER_KEY, LiveStream, LiveStreamSchema, $q, InstantiatorService, $filter) {
    $ionicModal.fromTemplateUrl('templates/shows/video.html', {
        scope: $rootScope
    }).then(function (modal) {
        $rootScope.modal = modal;
    });

    var showDataLive = null;
    var lastLivePermission = true;

    $rootScope.openVideoModal = function (showData, isLive) {
        if(isLive) {
            showDataLive = showData;
            checkVideoStreamPermissions();
        }else {
            playVideo(showData, false, null);
        }
    };

    function playVideo(showData, isLive, data) {
        $rootScope.playerSettings = {};
        $rootScope.playerSettings = {
            width: '100%',
            aspectratio: '16:9',
            androidhls: true
        };

        if (showData.title != undefined) $rootScope.playerSettings.title = showData.title;
        if (showData.OwnerTitle != undefined) $rootScope.playerSettings.title = showData.OwnerTitle;
        if (showData.CoverImage != undefined) $rootScope.playerSettings.image = showData.CoverImage;
        if (showData.MediaFileRef != undefined) $rootScope.playerSettings.file = showData.MediaFileRef;
        if (showData.Link != undefined) $rootScope.playerSettings.file = showData.Link;
        if (showData.Url != undefined) $rootScope.playerSettings.url = showData.Url;
        $rootScope.playerSettings.sharing = false;

        if(isLive) {
            var permission = data.status;
            $rootScope.playerSettings.file = data.stream;
            //$rootScope.playerSettings.file = 'http://ant1tvvod-vh.akamaihd.net/vods2/RR.mp4';
            //console.debug('permission', permission);
            //console.debug('data.stream', data.stream);

            if (!$rootScope.playerInstance) {
                initPlayer();
            }else {
                if(permission != lastLivePermission) {
                    $rootScope.playerInstance.setup($rootScope.playerSettings);
                    $rootScope.playerInstance.onReady(function () {
                        $rootScope.playerInstance.play();
                    });
                }
            }

            lastLivePermission = permission;

        } else {
            initPlayer();
        }

    };

    function initPlayer() {
        $rootScope.modal
            .show()
            .then(function () {
                jwplayer.key = JWPLAYER_KEY;
                $rootScope.playerInstance = jwplayer("video-player");
                $rootScope.playerInstance.setup($rootScope.playerSettings);
                $rootScope.playerInstance.onReady(function () {
                    $rootScope.playerInstance.play();
                });

            });
    }

    function checkVideoStreamPermissions() {
        var intervalInSeconds = 40;

        /**
         * Promises array
         *
         * @type {{schema_live: *}}
         */
        var promises = {
            /**
             * Gets the live stream schema object
             */
            schema_live: LiveStreamSchema.getLiveStream().then(function (value) {
                return InstantiatorService.instantiate(value.data).data;
            }, function (reason) {
                return [];
            }),
            /**
             * Gets the live stream
             */
            live: LiveStream.getLiveStream().then(function (value) {
                return value.data;
            }, function (reason) {
                return {};
            })
        };

        $q.all(promises).then(function (result) {
            /**
             * Merge schema objects with requested objects for always right structure
             * @type {Object}
             */
            var liveData = angular.merge([], result.schema_live, result.live);
            playVideo(showDataLive, true, liveData.data);

            if($rootScope.timeOutID) clearTimeout($rootScope.timeOutID);
            $rootScope.timeOutID = setTimeout(checkVideoStreamPermissions, intervalInSeconds * 1000);

        }).catch(function (reason) {

        }).finally(function () {

        });

    };

    /**
     * Function called from video modal to add current video to favorites
     * @param playerSettings
     */
    $rootScope.addToFavoritesFromVideo = function (playerSettings) {

    };

    $rootScope.closeModal = function () {
        clearTimeout($rootScope.timeOutID);
        showDataLive = null;
        $rootScope.playerInstance.remove();
        $rootScope.playerInstance = null;
        $rootScope.modal.hide();
    }
});